/*!
 * Consent tracking for Snowplow v4.6.3 ()
 * Copyright 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
 * Licensed under BSD-3-Clause
 */

(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.snowplowEnhancedConsentTracking = {}));
})(this, (function (exports) { 'use strict';

  /******************************************************************************
  Copyright (c) Microsoft Corporation.

  Permission to use, copy, modify, and/or distribute this software for any
  purpose with or without fee is hereby granted.

  THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
  REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
  AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
  INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
  LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
  PERFORMANCE OF THIS SOFTWARE.
  ***************************************************************************** */

  var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
  };

  function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
  }

  function __spreadArray(to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
  }

  typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
  };

  /*!
   * Core functionality for Snowplow JavaScript trackers v4.6.3 (http://bit.ly/sp-js)
   * Copyright 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
   * Licensed under BSD-3-Clause
   */

  /*
   * Copyright (c) 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
   * All rights reserved.
   *
   * Redistribution and use in source and binary forms, with or without
   * modification, are permitted provided that the following conditions are met:
   *
   * 1. Redistributions of source code must retain the above copyright notice, this
   *    list of conditions and the following disclaimer.
   *
   * 2. Redistributions in binary form must reproduce the above copyright notice,
   *    this list of conditions and the following disclaimer in the documentation
   *    and/or other materials provided with the distribution.
   *
   * 3. Neither the name of the copyright holder nor the names of its
   *    contributors may be used to endorse or promote products derived from
   *    this software without specific prior written permission.
   *
   * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
   * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
   * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
   * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   */
  function payloadBuilder() {
      var dict = {}, allJson = [], jsonForProcessing = [], contextEntitiesForProcessing = [];
      var processor;
      var add = function (key, value) {
          if (value != null && value !== '') {
              // null also checks undefined
              dict[key] = value;
          }
      };
      var addDict = function (dict) {
          for (var key in dict) {
              if (Object.prototype.hasOwnProperty.call(dict, key)) {
                  add(key, dict[key]);
              }
          }
      };
      var addJson = function (keyIfEncoded, keyIfNotEncoded, json) {
          if (json && isNonEmptyJson(json)) {
              var jsonWithKeys = { keyIfEncoded: keyIfEncoded, keyIfNotEncoded: keyIfNotEncoded, json: json };
              jsonForProcessing.push(jsonWithKeys);
              allJson.push(jsonWithKeys);
          }
      };
      var addContextEntity = function (entity) {
          contextEntitiesForProcessing.push(entity);
      };
      return {
          add: add,
          addDict: addDict,
          addJson: addJson,
          addContextEntity: addContextEntity,
          getPayload: function () { return dict; },
          getJson: function () { return allJson; },
          withJsonProcessor: function (jsonProcessor) {
              processor = jsonProcessor;
          },
          build: function () {
              processor === null || processor === void 0 ? void 0 : processor(this, jsonForProcessing, contextEntitiesForProcessing);
              return dict;
          },
      };
  }
  /**
   * Is property a non-empty JSON?
   * @param property - Checks if object is non-empty json
   */
  function isNonEmptyJson(property) {
      if (!isJson(property)) {
          return false;
      }
      for (var key in property) {
          if (Object.prototype.hasOwnProperty.call(property, key)) {
              return true;
          }
      }
      return false;
  }
  /**
   * Is property a JSON?
   * @param property - Checks if object is json
   */
  function isJson(property) {
      return (typeof property !== 'undefined' &&
          property !== null &&
          (property.constructor === {}.constructor || property.constructor === [].constructor));
  }

  /*
   * Copyright (c) 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
   * All rights reserved.
   *
   * Redistribution and use in source and binary forms, with or without
   * modification, are permitted provided that the following conditions are met:
   *
   * 1. Redistributions of source code must retain the above copyright notice, this
   *    list of conditions and the following disclaimer.
   *
   * 2. Redistributions in binary form must reproduce the above copyright notice,
   *    this list of conditions and the following disclaimer in the documentation
   *    and/or other materials provided with the distribution.
   *
   * 3. Neither the name of the copyright holder nor the names of its
   *    contributors may be used to endorse or promote products derived from
   *    this software without specific prior written permission.
   *
   * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
   * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
   * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
   * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
   * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
   * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
   * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   */
  var label = 'Snowplow: ';
  var LOG_LEVEL;
  (function (LOG_LEVEL) {
      LOG_LEVEL[LOG_LEVEL["none"] = 0] = "none";
      LOG_LEVEL[LOG_LEVEL["error"] = 1] = "error";
      LOG_LEVEL[LOG_LEVEL["warn"] = 2] = "warn";
      LOG_LEVEL[LOG_LEVEL["debug"] = 3] = "debug";
      LOG_LEVEL[LOG_LEVEL["info"] = 4] = "info";
  })(LOG_LEVEL || (LOG_LEVEL = {}));
  var LOG = logger();
  function logger(logLevel) {
      if (logLevel === void 0) { logLevel = LOG_LEVEL.warn; }
      function setLogLevel(level) {
          if (LOG_LEVEL[level]) {
              logLevel = level;
          }
          else {
              logLevel = LOG_LEVEL.warn;
          }
      }
      /**
       * Log errors, with or without error object
       */
      function error(message, error) {
          var extraParams = [];
          for (var _i = 2; _i < arguments.length; _i++) {
              extraParams[_i - 2] = arguments[_i];
          }
          if (logLevel >= LOG_LEVEL.error && typeof console !== 'undefined') {
              var logMsg = label + message + '\n';
              if (error) {
                  console.error.apply(console, __spreadArray([logMsg + '\n', error], extraParams, false));
              }
              else {
                  console.error.apply(console, __spreadArray([logMsg], extraParams, false));
              }
          }
      }
      /**
       * Log warnings, with or without error object
       */
      function warn(message, error) {
          var extraParams = [];
          for (var _i = 2; _i < arguments.length; _i++) {
              extraParams[_i - 2] = arguments[_i];
          }
          if (logLevel >= LOG_LEVEL.warn && typeof console !== 'undefined') {
              var logMsg = label + message;
              if (error) {
                  console.warn.apply(console, __spreadArray([logMsg + '\n', error], extraParams, false));
              }
              else {
                  console.warn.apply(console, __spreadArray([logMsg], extraParams, false));
              }
          }
      }
      /**
       * Log debug messages
       */
      function debug(message) {
          var extraParams = [];
          for (var _i = 1; _i < arguments.length; _i++) {
              extraParams[_i - 1] = arguments[_i];
          }
          if (logLevel >= LOG_LEVEL.debug && typeof console !== 'undefined') {
              console.debug.apply(console, __spreadArray([label + message], extraParams, false));
          }
      }
      /**
       * Log info messages
       */
      function info(message) {
          var extraParams = [];
          for (var _i = 1; _i < arguments.length; _i++) {
              extraParams[_i - 1] = arguments[_i];
          }
          if (logLevel >= LOG_LEVEL.info && typeof console !== 'undefined') {
              console.info.apply(console, __spreadArray([label + message], extraParams, false));
          }
      }
      return { setLogLevel: setLogLevel, warn: warn, error: error, debug: debug, info: info };
  }
  /**
   * Build a self-describing event
   * A custom event type, allowing for an event to be tracked using your own custom schema
   * and a data object which conforms to the supplied schema
   *
   * @param event - Contains the properties and schema location for the event
   * @returns PayloadBuilder to be sent to {@link @snowplow/tracker-core#TrackerCore.track}
   */
  function buildSelfDescribingEvent(event) {
      var _a = event.event, schema = _a.schema, data = _a.data, pb = payloadBuilder();
      var ueJson = {
          schema: 'iglu:com.snowplowanalytics.snowplow/unstruct_event/jsonschema/1-0-0',
          data: { schema: schema, data: data },
      };
      pb.add('e', 'ue');
      pb.addJson('ue_px', 'ue_pr', ueJson);
      return pb;
  }

  /*!
   * Core functionality for Snowplow Browser trackers v4.6.3 (http://bit.ly/sp-js)
   * Copyright 2022 Snowplow Analytics Ltd, 2010 Anthon Pang
   * Licensed under BSD-3-Clause
   */
  /**
   * Dispatch function to all specified trackers from the supplied collection
   *
   * @param trackers - An optional list of trackers to send the event to, or will send to all trackers
   * @param trackerCollection - The collection which the trackers will be selected from
   * @param fn - The function which will run against each tracker
   */
  function dispatchToTrackersInCollection(trackers, trackerCollection, fn) {
      try {
          getTrackersFromCollection(trackers !== null && trackers !== void 0 ? trackers : Object.keys(trackerCollection), trackerCollection).forEach(fn);
      }
      catch (ex) {
          LOG.error('Function failed', ex);
      }
  }
  function getTrackersFromCollection(trackerIds, trackerCollection) {
      var trackers = [];
      for (var _i = 0, trackerIds_1 = trackerIds; _i < trackerIds_1.length; _i++) {
          var id = trackerIds_1[_i];
          if (trackerCollection.hasOwnProperty(id)) {
              trackers.push(trackerCollection[id]);
          }
          else {
              LOG.warn(id + ' not configured');
          }
      }
      return trackers;
  }

  var CONSENT_SCHEMA = 'iglu:com.snowplowanalytics.snowplow/consent_preferences/jsonschema/1-0-0';
  var CMP_VISIBLE_SCHEMA = 'iglu:com.snowplowanalytics.snowplow/cmp_visible/jsonschema/1-0-0';

  /**
   * Build a Consent Action Event
   * for tracking consent actions
   *
   * @param event - Contains properties of Consent action events
   * @returns PayloadBuilder to be sent to {@link @snowplow/tracker-core#TrackerCore.track}
   */
  function buildConsentEvent(event) {
      return buildSelfDescribingEvent({
          event: {
              schema: CONSENT_SCHEMA,
              data: removeEmptyProperties(__assign({}, event)),
          },
      });
  }
  /**
   * Build a CMP Visible Event
   * for tracking loadtime of CMP banner
   *
   * @param event - Contains properties of CMP Visible events
   * @returns PayloadBuilder to be sent to {@link @snowplow/tracker-core#TrackerCore.track}
   */
  function buildCmpVisibleEvent(event) {
      return buildSelfDescribingEvent({
          event: {
              schema: CMP_VISIBLE_SCHEMA,
              data: removeEmptyProperties(__assign({}, event)),
          },
      });
  }
  /**
   * Returns a copy of a JSON with undefined and null properties removed
   *
   * @param event - Object to clean
   * @returns A cleaned copy of eventJson
   */
  function removeEmptyProperties(event) {
      var ret = {};
      for (var k in event) {
          if (event[k] !== null && typeof event[k] !== 'undefined') {
              ret[k] = event[k];
          }
      }
      return ret;
  }

  var _trackers = {};
  /**
   * Adds consent tracking
   */
  function EnhancedConsentPlugin() {
      var trackerId;
      return {
          activateBrowserPlugin: function (tracker) {
              trackerId = tracker.id;
              _trackers[trackerId] = tracker;
          },
      };
  }
  /**
   * Track the loadtime of a CMP banner
   */
  function trackCmpVisible(cmpVisible, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      var context = cmpVisible.context, timestamp = cmpVisible.timestamp, elapsedTime = cmpVisible.elapsedTime;
      dispatchToTrackersInCollection(trackers, _trackers, function (t) {
          t.core.track(buildCmpVisibleEvent({
              elapsedTime: elapsedTime,
          }), context, timestamp);
      });
  }
  // Generic consent action
  function trackConsentAction(consent, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      var context = consent.context, timestamp = consent.timestamp, consentAttributes = __rest(consent, ["context", "timestamp"]);
      dispatchToTrackersInCollection(trackers, _trackers, function (t) {
          t.core.track(buildConsentEvent(consentAttributes), context, timestamp);
      });
  }
  /**
   * Track an allow consent event
   *
   * @param consent - The consent information
   * @param trackers - The tracker identifiers which the event will be sent to
   */
  function trackConsentAllow(consent, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      trackConsentAction(__assign(__assign({}, consent), { eventType: 'allow_all' }), trackers);
  }
  /**
   * Track an allow selected consent event
   *
   * @param consent - The consent information
   * @param trackers - The tracker identifiers which the event will be sent to
   */
  function trackConsentSelected(consent, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      trackConsentAction(__assign(__assign({}, consent), { eventType: 'allow_selected' }), trackers);
  }
  /**
   * Track a consent pending event
   *
   * @param consent - The consent information
   * @param trackers - The tracker identifiers which the event will be sent to
   */
  function trackConsentPending(consent, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      trackConsentAction(__assign(__assign({}, consent), { eventType: 'pending' }), trackers);
  }
  /**
   * Track an implicit consent event
   *
   * @param consent - The consent information
   * @param trackers - The tracker identifiers which the event will be sent to
   */
  function trackConsentImplicit(consent, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      trackConsentAction(__assign(__assign({}, consent), { eventType: 'implicit_consent' }), trackers);
  }
  /**
   * Track a deny consent event
   *
   * @param consent - The consent information
   * @param trackers - The tracker identifiers which the event will be sent to
   */
  function trackConsentDeny(consent, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      trackConsentAction(__assign(__assign({}, consent), { eventType: 'deny_all' }), trackers);
  }
  /**
   * Track a consent expired event
   *
   * @param consent - The consent information
   * @param trackers - The tracker identifiers which the event will be sent to
   */
  function trackConsentExpired(consent, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      trackConsentAction(__assign(__assign({}, consent), { eventType: 'expired' }), trackers);
  }
  /**
   * Track a consent withdrawn event
   *
   * @param consent - The consent information
   * @param trackers - The tracker identifiers which the event will be sent to
   */
  function trackConsentWithdrawn(consent, trackers) {
      if (trackers === void 0) { trackers = Object.keys(_trackers); }
      trackConsentAction(__assign(__assign({}, consent), { eventType: 'withdrawn' }), trackers);
  }

  exports.EnhancedConsentPlugin = EnhancedConsentPlugin;
  exports.trackCmpVisible = trackCmpVisible;
  exports.trackConsentAllow = trackConsentAllow;
  exports.trackConsentDeny = trackConsentDeny;
  exports.trackConsentExpired = trackConsentExpired;
  exports.trackConsentImplicit = trackConsentImplicit;
  exports.trackConsentPending = trackConsentPending;
  exports.trackConsentSelected = trackConsentSelected;
  exports.trackConsentWithdrawn = trackConsentWithdrawn;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=index.umd.js.map
